"""This file contains code to interact with docker api"""

from typing import Optional

import click
import docker
from docker import DockerClient
from docker.errors import ImageNotFound
from docker.models.images import Image


def get_client() -> DockerClient:
    """Return docker client instance for interacting with docker api."""
    return docker.from_env()


def get_image(docker_client: DockerClient, name: str, pull: bool) -> Optional[Image]:
    """Return Image object."""
    image = None

    try:
        if pull:
            click.echo(f"Pulling image {name}")
            docker_client.images.pull(name)

        image = docker_client.images.get(name)
    except ImageNotFound as image_not_found:
        click.echo(f"{ImageNotFound.__name__}: {image_not_found}", err=True)
        if not pull:
            click.echo("Did you want to --pull the image?")

    return image



"""This module contains methods to reconstruct Dockerfile from available data."""

from dockerfile_recover.parser import parse_history


def generate_dockerfile(docker_history: list[str]):
    """Return generated Dockerfile."""
    docker_history.reverse()
    return parse_history(docker_history)



"""This module contains functions for parsing NOP commands from docker history."""


def get_nop() -> str:
    """Return marker for NOP, which represents a docker command."""
    return "/bin/sh -c #(nop)"


def is_nop(command: str) -> bool:
    """Check if command is NOP."""
    return command.startswith(get_nop())


def strip_nop(command: str) -> str:
    """Return command with NOP marker removed."""
    return command.removeprefix(get_nop())



"""This module contains functions for parsing shell commands from docker history."""


def get_shell() -> str:
    """Return marker for shell, which can be a NOP docker command or shell command."""
    return "/bin/sh -c"


def is_shell(command: str) -> bool:
    """Check if command is shell."""
    return command.startswith(get_shell())


def strip_shell(command: str) -> str:
    """Return command with shell marker removed."""
    return command.removeprefix(get_shell())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dockerfile_recover', 'dockerfile_recover.parser']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.1,<9.0.0', 'docker>=5.0.0,<6.0.0', 'six>=1.16.0,<2.0.0']

entry_points = \
{'console_scripts': ['dockerfile-recover = dockerfile_recover.__main__:main']}

setup_kwargs = {
    'name': 'dockerfile-recover',
    'version': '0.0.1a6',
    'description': 'dockerfile-recover is a tool to reconstruct Dockerfile by reverse engineering a docker image',
    'long_description': '# dockerfile-recover\ndockerfile-recover is a tool to reconstruct Dockerfile by reverse engineering a docker image\n\n# Installation\n\n## With pip\n```\npip install dockerfile-recover\n```\n## In a container\n```\ndocker pull purificant/dockerfile-recover\n```\n\n# Usage\n\n## When installing with pip\n```\ndockerfile-recover recover IMAGE_NAME\n```\nOptionally `--pull` the image, for example:\n```\ndockerfile-recover recover --pull python\ndockerfile-recover recover --pull django\ndockerfile-recover recover --pull redis\ndockerfile-recover recover --pull node\n```\n\nor run as python module\n```\npython -m dockerfile_recover recover --pull nginx\n```\n\n## When running in a container\n\n```\ndocker run -it -v /var/run/docker.sock:/var/run/docker.sock purificant/dockerfile-recover python -m dockerfile_recover recover IMAGE_NAME\n```\n',
    'author': None,
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/purificant/dockerfile-recover',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



"""This module contains main codebase for dockerfile-recover tool."""

import click
import docker
from docker.errors import APIError
from docker.models.images import Image

from dockerfile_recover.docker_api import get_client, get_image
from dockerfile_recover.generator import generate_dockerfile


@click.group()
def cli():
    """This tool reconstructs Dockerfile by reverse engineering a docker image."""


@click.command()
@click.option(
    "--pull",
    is_flag=True,
    default=False,
    help="Pull the image first. Default is to search for image in `docker images` without pulling.",
)
@click.argument("image_name")
def recover(image_name: str, pull: bool) -> None:
    """Recover Dockerfile given an image name, searches `docker images` to find the image,
    or can optionally --pull it first."""

    try:

        docker_client: docker.DockerClient = get_client()

        image: Image = get_image(docker_client, image_name, pull)
        if not isinstance(image, Image):
            return

        lines = generate_dockerfile(image.history())

        click.echo("Recovered Dockerfile:")
        click.echo("")
        for line in lines:
            click.echo(line)

    except APIError as api_error:
        click.echo(f"{APIError.__name__}: {api_error}", err=True)


# use full help text in place of short_help
recover.short_help = recover.__doc__
cli.add_command(recover)
cli()
